@@ -2,6 +2,10 @@ import datetime as dt
 import requests
 from typing import List, Dict
 import logging
+import random
+from collections import Counter
+import numpy as np 
+from time import sleep
 
 from .provider import ContentProvider
 from .exceptions import UnsupportedOperationException
@@ -15,13 +19,17 @@ class TwitterTwitterProvider(ContentProvider):
     """
     All these endpoints accept a `usernames: List[str]` keyword arg.
     """
-
+    
+    MAX_QUERY_LENGTH = 1024 #I think?
+    POLITENESS_DELAY = 1 #sleep for half a second if we're gonna spam a bunch of queries
+    
     def __init__(self, bearer_token=None):
         super(TwitterTwitterProvider, self).__init__()
         self._logger = logging.getLogger(__name__)
         self._bearer_token = bearer_token
         self._session = requests.Session()  # better performance to put all HTTP through this one object
 
+    #Chunk'd
     def sample(self, query: str, start_date: dt.datetime, end_date: dt.datetime, limit: int = 10, **kwargs) -> List[Dict]:
         """
         Return a list of historical tweets matching the query.
@@ -33,16 +41,27 @@ class TwitterTwitterProvider(ContentProvider):
         :return:
         """
         # sample of historical tweets
-        params = {
-            "query": self._assembled_query_str(query, **kwargs),
-            "max_results": limit,
-            "start_time": start_date.isoformat("T") + "Z",
-            "end_time": self._fix_end_date(end_date).isoformat("T") + "Z",
-            "tweet.fields": ",".join(["author_id", "created_at", "public_metrics"]),
-            "expansions": "author_id",
-        }
-        results = self._cached_query("tweets/search/all", params)
-        return TwitterTwitterProvider._tweets_to_rows(results)
+        all_results = []
+        chunks = self._assemble_and_chunk_query_str(query, **kwargs)
+        for subquery in chunks:
+            params = {
+                "query": subquery,
+                "max_results": limit,
+                "start_time": start_date.isoformat("T") + "Z",
+                "end_time": self._fix_end_date(end_date).isoformat("T") + "Z",
+                "tweet.fields": ",".join(["author_id", "created_at", "public_metrics"]),
+                "expansions": "author_id",
+            }
+            results = self._cached_query("tweets/search/all", params)
+            results = TwitterTwitterProvider._tweets_to_rows(results)
+            all_results.extend(results)
+            if len(chunks) > 1:
+                
+                sleep(self.POLITENESS_DELAY)
+                
+        if len(all_results) > limit:
+            all_results = random.sample(all_results, limit) 
+        return results
 
     @classmethod
     def _assembled_query_str(cls, query: str, **kwargs) -> str:
@@ -54,16 +73,46 @@ class TwitterTwitterProvider(ContentProvider):
             assembled_query = query + " (" + " OR ".join(["from:{}".format(name) for name in usernames]) + ")"
         # check if query too long
         # @see https://developer.twitter.com/en/docs/twitter-api/tweets/search/api-reference/get-tweets-search-all
-        if len(assembled_query) > 1024:
-            raise RuntimeError("Twitter's max query length is 1024 characters - your query is {} characters. "
-                               "Try changing collections.".format(len(assembled_query)))
+        #if len(assembled_query) > 1024:
+        #    raise RuntimeError("Twitter's max query length is 1024 characters - your query is {} characters. "
+        #                       "Try changing collections.".format(len(assembled_query)))
         return assembled_query
 
+    
+    @classmethod
+    def _assemble_and_chunk_query_str(cls, base_query: str, **kwargs) -> List:
+        """
+        If a query string is too long, we can attempt to run it anyway by splitting the domain substring (which is guaranteed 
+        too be only a sequence of ANDs) into parts, to produce multiple smaller queries which are collectively equivalent 
+        to the original. 
+        """
+        usernames = kwargs.get('usernames', [])
+        
+        if len(base_query) > cls.MAX_QUERY_LENGTH:
+            ##of course there still is the possibility that the base query is too large, which 
+            #cannot be fixed by this method
+            raise RuntimeError(f"Base Query cannot exceed {cls.MAX_QUERY_LENGTH} characters")
+        
+        queries = [cls._assembled_query_str(base_query, usernames=usernames)]
+        queries_too_big = any([len(q_) > cls.MAX_QUERY_LENGTH for q_ in queries])
+        username_divisor = 2
+        
+        if queries_too_big:
+            while queries_too_big:
+                chunked_users = np.array_split(usernames, username_divisor)
+                queries = [cls._assembled_query_str(base_query, usernames=users) for users in chunked_users]
+                queries_too_big = any([len(q_) > cls.MAX_QUERY_LENGTH for q_ in queries])
+                username_divisor *= 2
+            
+        return queries
+    
+    #No need to chunk
     def count(self, query: str, start_date: dt.datetime, end_date: dt.datetime, **kwargs) -> int:
         results = self.count_over_time(query, start_date, end_date, **kwargs)  # use the cached counts being made already
         total = sum([r['count'] for r in results['counts']])
         return total
-
+    
+    #Chunk
     def count_over_time(self, query: str, start_date: dt.datetime,
                         end_date: dt.datetime,
                         **kwargs) -> Dict:
@@ -75,65 +124,80 @@ class TwitterTwitterProvider(ContentProvider):
         :param kwargs:
         :return:
         """
-        params = dict(
-            query=self._assembled_query_str(query, **kwargs),
-            granularity='day',
-            start_time=start_date.isoformat("T") + "Z",
-            end_time=self._fix_end_date(end_date).isoformat("T") + "Z",
-        )
-        more_data = True
-        next_token = None
-        data = []
-        while more_data:
-            params['next_token'] = next_token
-            results = self._cached_query("tweets/counts/all", params)
-            data += reversed(results['data'])
-            if ('meta' in results) and ('next_token' in results['meta']):
-                next_token = results['meta']['next_token']
-                more_data = True
-            else:
-                next_token = None
-                more_data = False
+        counter = Counter()
+
+        for subquery in  self._assemble_and_chunk_query_str(query, **kwargs):
+            #print(subquery)
+            params = dict(
+                query=subquery,
+                granularity='day',
+                start_time=start_date.isoformat("T") + "Z",
+                end_time=self._fix_end_date(end_date).isoformat("T") + "Z",
+            )
+            more_data = True
+            next_token = None
+            data = []
+            while more_data:
+                params['next_token'] = next_token
+                results = self._cached_query("tweets/counts/all", params)
+                data += reversed(results['data'])
+                if ('meta' in results) and ('next_token' in results['meta']):
+                    next_token = results['meta']['next_token']
+                    more_data = True
+                else:
+                    next_token = None
+                    more_data = False
+            countable = {i["start"]:i["tweet_count"] for i in data}
+            counter += Counter(countable)
+                
+        data = dict(counter)
+        
         to_return = []
-        for d in data:
+        for date, count in data.items():
             to_return.append({
-                'date': dt.datetime.strptime(d['start'], TWITTER_DATE_FORMAT),
-                'timestamp': dt.datetime.strptime(d['start'], TWITTER_DATE_FORMAT).timestamp(),
-                'count': d['tweet_count'],
+                'date': dt.datetime.strptime(date, TWITTER_DATE_FORMAT),
+                'timestamp': dt.datetime.strptime(date, TWITTER_DATE_FORMAT).timestamp(),
+                'count': count,
             })
         return {'counts': to_return}
 
+    #Chunk'd
     def all_items(self, query: str, start_date: dt.datetime, end_date: dt.datetime, page_size: int = 500,
                   **kwargs):
-        limit = kwargs['limit'] if 'limit' in kwargs else None
-        next_token = None
-        more_data = True
-        params = {
-            "query": self._assembled_query_str(query, **kwargs),
-            "max_results": page_size,
-            "start_time": start_date.isoformat("T") + "Z",
-            "end_time": self._fix_end_date(end_date).isoformat("T") + "Z",
-            "tweet.fields": ",".join(["author_id", "created_at", "public_metrics"]),
-            "expansions": "author_id",
-        }
-        item_count = 0
-        while more_data and ((limit is not None) and (item_count < limit)):
-            params['next_token'] = next_token
-            results = self._cached_query("tweets/search/all", params)
-            result_count = results['meta']['result_count']
-            if result_count == 0:
-                more_data = False
-                continue
-            page = TwitterTwitterProvider._tweets_to_rows(results)
-            yield page
-            item_count += len(page)
-            next_token = results['meta']['next_token'] if 'next_token' in results['meta'] else None
-            more_data = next_token is not None
-
+        
+        for subquery in self._assemble_and_chunk_query_str(query, **kwargs):
+            
+            limit = kwargs['limit'] if 'limit' in kwargs else None
+            next_token = None
+            more_data = True
+            params = {
+                "query": subquery,
+                "max_results": page_size,
+                "start_time": start_date.isoformat("T") + "Z",
+                "end_time": self._fix_end_date(end_date).isoformat("T") + "Z",
+                "tweet.fields": ",".join(["author_id", "created_at", "public_metrics"]),
+                "expansions": "author_id",
+            }
+            item_count = 0
+            while more_data and ((limit is not None) and (item_count < limit)):
+                params['next_token'] = next_token
+                results = self._cached_query("tweets/search/all", params)
+                result_count = results['meta']['result_count']
+                if result_count == 0:
+                    more_data = False
+                    continue
+                page = TwitterTwitterProvider._tweets_to_rows(results)
+                yield page
+                item_count += len(page)
+                next_token = results['meta']['next_token'] if 'next_token' in results['meta'] else None
+                more_data = next_token is not None
+    
+    #sampled_languages just relies on all_items, so if all_items is chunked then we get this for free
     def languages(self, query: str, start_date: dt.datetime, end_date: dt.datetime, limit: int = 10, **kwargs) -> List[Dict]:
         # use the helper because we need to sample from most recent tweets
         return self._sampled_languages(query, start_date, end_date, limit, **kwargs)
-
+    
+    #same as languages
     def words(self, query: str, start_date: dt.datetime, end_date: dt.datetime, limit: int = 100,
               **kwargs) -> List[Dict]:
         # use the helper because we need to sample from most recent tweets
